// This is the source file where general methods of
// BlackHawk are implemented.
// Last modification: 14 November 2019
// Authors: Jérémy Auffinger j.auffinger@ipnl.in2p3.fr & Alexandre Arbey alexandre.arbey@ens-lyon.fr

#include "include.h"

// *******GENERAL FUNCTIONS*******
int read_params(struct param *parameters, char name[],int session){
	// This function reads the file 'parameters.txt' and fills
	// the parameters structure. It updates the value of success
	// to 1 (no error in the parameters) or 0 (errors)
	
	FILE *param_file;
	param_file = fopen(name,"r");
	if(!param_file){
		printf("\n\t [read_params] : ERROR COULD NOT OPEN FILE '%s' !\n",name);
		fflush(stdout);
		exit(0);
	}
	rewind(param_file);
	
	char dummy[512];	// contains irrelevant characters of the parameters file (2 for each parameter, 1st is its name and 2nd is =)
	
	double Emin_hadro_PYTHIA,Emax_hadro_PYTHIA;
	int nb_fin_en_PYTHIA,nb_init_en_PYTHIA,nb_init_part_PYTHIA,nb_fin_part_PYTHIA;
	double Emin_hadro_HERWIG,Emax_hadro_HERWIG;
	int nb_fin_en_HERWIG,nb_init_en_HERWIG,nb_init_part_HERWIG,nb_fin_part_HERWIG;
	double Emin_hadro_PYTHIA_new,Emax_hadro_PYTHIA_new;
	int nb_fin_en_PYTHIA_new,nb_init_en_PYTHIA_new,nb_init_part_PYTHIA_new,nb_fin_part_PYTHIA_new;
	
	Emin_hadro_PYTHIA=Emax_hadro_PYTHIA=Emin_hadro_HERWIG=Emax_hadro_HERWIG=Emin_hadro_PYTHIA_new=Emax_hadro_PYTHIA_new=0.;
	nb_fin_en_PYTHIA=nb_init_en_PYTHIA=nb_init_part_PYTHIA=nb_fin_part_PYTHIA=nb_fin_en_HERWIG=nb_init_en_HERWIG=nb_init_part_HERWIG=nb_fin_part_HERWIG=nb_fin_en_PYTHIA_new=nb_init_en_PYTHIA_new=nb_init_part_PYTHIA_new=nb_fin_part_PYTHIA_new=0;	
	
	double amplitude_lognormal,stand_dev_lognormal,crit_mass_lognormal,amplitude_powerlaw,eqstate_powerlaw,amplitude_critical_collapse,crit_mass_critical_collapse,amplitude_uniform;
	double stand_dev_a_gaussian,mean_a_gaussian;
	
	amplitude_lognormal=stand_dev_lognormal=crit_mass_lognormal=amplitude_powerlaw=eqstate_powerlaw=amplitude_critical_collapse=crit_mass_critical_collapse=amplitude_uniform=0.;
	stand_dev_a_gaussian=mean_a_gaussian=0.;
	
	while(EOF != fscanf(param_file,"%s",dummy))
	{
		if(!strncasecmp("#",dummy,1)) while ((EOF!=fscanf(param_file,"%c",dummy))&&(strncasecmp("\n",dummy,1)));
		if(!strcasecmp(dummy,"destination_folder"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%s",parameters->destination_folder);
		}
		else if(!strcasecmp(dummy,"full_output"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%i",&(parameters->full_output));
			
			if(parameters->full_output != 0 && parameters->full_output != 1)
			{
				printf("\n\t [read_params] : ERROR WRONG OUTPUT CHOICE !\n");
				fflush(stdout);
				fclose(param_file);
				return 0;
			}
		}
		else if(!strcasecmp(dummy,"interpolation_method"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%i",&(parameters->interpolation_method));
			
			if(parameters->interpolation_method != 0 && parameters->interpolation_method != 1)
			{
				printf("\n\t [read_params] : ERROR WRONG INTERPOLATION METHOD !\n");
				fflush(stdout);
				fclose(param_file);
				return 0;
			}
		}
		else if(!strcasecmp(dummy,"BHnumber"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%i",&(parameters->BHnumber));
			if(parameters->BHnumber < 1)
			{
				printf("\n\t [read_params] : ERROR WRONG BLACK HOLE INITIAL MASSES NUMBER !\n");
				fflush(stdout);
				fclose(param_file);
				return 0;
			}
		}
		else if(!strcasecmp(dummy,"Mmin"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%lf",&(parameters->Mmin));
			parameters->Mmin = parameters->Mmin*mass_conversion; // conversion from CGS to GeV
			
			if(parameters->Mmin < 0.)
			{
				printf("\n\t [read_params] : ERROR WRONG BLACK HOLE MASS BOUNDARIES !\n");
				fflush(stdout);
				fclose(param_file);
				return 0;
			}			
		}
		else if(!strcasecmp(dummy,"Mmax"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%lf",&(parameters->Mmax));
			parameters->Mmax = parameters->Mmax*mass_conversion; // conversion from CGS to GeV
		}
		else if(!strcasecmp(dummy,"anumber"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%i",&(parameters->anumber));
			if(parameters->anumber < 1)
			{
				printf("\n\t [read_params] : ERROR WRONG BLACK HOLE INITIAL SPINS NUMBER !\n");
				fflush(stdout);
				fclose(param_file);
				return 0;
			}
		}
		else if(!strcasecmp(dummy,"amin"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%lf",&(parameters->amin));
		}
		else if(!strcasecmp(dummy,"amax"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%lf",&(parameters->amax));
		}
		else if(!strcasecmp(dummy,"spectrum_choice"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%i",&(parameters->spectrum_choice));
			
			if(parameters->spectrum_choice < 0 || parameters->spectrum_choice > 5)
			{
				printf("\n\t [read_params] : ERROR WRONG MASS SPECTRUM CHOICE !\n");
				fflush(stdout);
				fclose(param_file);
				return 0;
			}
		}
		else if(!strcasecmp(dummy,"spectrum_choice_a"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%i",&(parameters->spectrum_choice_a));
			
			if(parameters->spectrum_choice_a < 0 || parameters->spectrum_choice_a > 2)
			{
				printf("\n\t [read_params] : ERROR WRONG SPIN SPECTRUM CHOICE !\n");
				fflush(stdout);
				fclose(param_file);
				return 0;
			}
		}
		else if(!strcasecmp(dummy,"amplitude_lognormal"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%lf",&(amplitude_lognormal));

		}
		else if(!strcasecmp(dummy,"stand_dev_lognormal"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%lf",&(stand_dev_lognormal));	
		}
		else if(!strcasecmp(dummy,"crit_mass_lognormal"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%lf",&(crit_mass_lognormal));
			
		}
		else if(!strcasecmp(dummy,"amplitude_powerlaw"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%lf",&(amplitude_powerlaw));

		}
		else if(!strcasecmp(dummy,"eqstate_powerlaw"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%lf",&(eqstate_powerlaw));
		}
		else if(!strcasecmp(dummy,"amplitude_critical_collapse"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%lf",&(amplitude_critical_collapse));
		}
		else if(!strcasecmp(dummy,"crit_mass_critical_collapse"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%lf",&crit_mass_critical_collapse);
		}
		else if(!strcasecmp(dummy,"amplitude_uniform"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%lf",&amplitude_uniform);
		}
		else if(!strcasecmp(dummy,"stand_dev_a_gaussian"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%lf",&stand_dev_a_gaussian);
		}
		else if(!strcasecmp(dummy,"mean_a_gaussian"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%lf",&mean_a_gaussian);
		}
		else if(!strcasecmp(dummy,"table"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%s",parameters->table);
		}
		else if(!strcasecmp(dummy,"tmin"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%lf",&(parameters->tmin));
			
			if(parameters->tmin < 0.)
			{
				printf("\n\t [read_params] : ERROR WRONG INITIAL TIME FOR BLACK HOLES EVOLUTION !\n");
				fflush(stdout);
				fclose(param_file);
				return 0;
			}	
			parameters->tmin = parameters->tmin*time_conversion; // conversion from CGS to GeV
		}
		else if(!strcasecmp(dummy,"limit"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%i",&(parameters->limit));
			
			if(parameters->limit < 1)
			{
				printf("\n\t [read_params] : ERROR WRONG RECURSION LIMIT !\n");
				fflush(stdout);
				fclose(param_file);
				return 0;
			}
			parameters->limit = parameters->limit*parameters->BHnumber*parameters->anumber;
		}
		/*else if(!strcasecmp(dummy,"Mmin_fM"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%lf",&(parameters->Mmin_fM));
			parameters->Mmin_fM = parameters->Mmin_fM*mass_conversion;
		}
		else if(!strcasecmp(dummy,"Mmax_fM"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%lf",&(parameters->Mmax_fM));
			parameters->Mmax_fM = parameters->Mmax_fM*mass_conversion;
		}
		else if(!strcasecmp(dummy,"amin_fM"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%lf",&(parameters->amin_fM));
		}
		else if(!strcasecmp(dummy,"amax_fM"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%lf",&(parameters->amax_fM));
		}
		else if(!strcasecmp(dummy,"nb_fM_masses"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%i",&(parameters->nb_fM_masses));
		}
		else if(!strcasecmp(dummy,"nb_fM_a"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%i",&(parameters->nb_fM_a));
		}*/
		else if(!strcasecmp(dummy,"Enumber"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%i",&(parameters->Enumber));
			
			if(parameters->Enumber < 1)
			{
				printf("\n\t [read_params] : ERROR WRONG NUMBER OF PRIMARY EMISSION ENERGIES !\n");
				fflush(stdout);
				fclose(param_file);
				return 0;
			}
		}
		else if(!strcasecmp(dummy,"Emin"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%lf",&(parameters->Emin));
		}
		else if(!strcasecmp(dummy,"Emax"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%lf",&(parameters->Emax));
		}
		/*else if(!strcasecmp(dummy,"particle_number"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%i",&(parameters->particle_number));
		}*/
		else if(!strcasecmp(dummy,"grav"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%i",&(parameters->grav));
			
			if(parameters->grav != 0 && parameters->grav != 1)
			{
				printf("\n\t [read_params] : ERROR WRONG GRAVITON CHOICE !\n");
				fflush(stdout);
				fclose(param_file);
				return 0;
			}
		}
		/*else if(!strcasecmp(dummy,"nb_gamma_a"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%i",&(parameters->nb_gamma_a));
		}
		else if(!strcasecmp(dummy,"nb_gamma_x"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%i",&(parameters->nb_gamma_x));
		}*/
		else if(!strcasecmp(dummy,"primary_only"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%i",&(parameters->primary_only));
		}
		else if(!strcasecmp(dummy,"hadronization_choice"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%i",&(parameters->hadronization_choice));
			
			if(parameters->hadronization_choice < 0 || parameters->hadronization_choice > 2)
			{
				printf("\n\t [read_params] : ERROR WRONG HADRONIZATION CHOICE !\n");
				fflush(stdout);
				fclose(param_file);
				return 0;
			}
		}
		/*else if(!strcasecmp(dummy,"Emin_hadro_PYTHIA"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%lf",&(Emin_hadro_PYTHIA));
		}
		else if(!strcasecmp(dummy,"Emax_hadro_PYTHIA"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%lf",&(Emax_hadro_PYTHIA));
		}
		else if(!strcasecmp(dummy,"nb_init_en_PYTHIA"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%i",&(nb_init_en_PYTHIA));
		}
		else if(!strcasecmp(dummy,"nb_fin_en_PYTHIA"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%i",&(nb_fin_en_PYTHIA));
		}
		else if(!strcasecmp(dummy,"nb_init_part_PYTHIA"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%i",&(nb_init_part_PYTHIA));
		}
		else if(!strcasecmp(dummy,"nb_fin_part_PYTHIA"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%i",&(nb_fin_part_PYTHIA));
		}
		else if(!strcasecmp(dummy,"Emin_hadro_HERWIG"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%lf",&(Emin_hadro_HERWIG));
		}
		else if(!strcasecmp(dummy,"Emax_hadro_HERWIG"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%lf",&(Emax_hadro_HERWIG));
		}
		else if(!strcasecmp(dummy,"nb_init_en_HERWIG"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%i",&(nb_init_en_HERWIG));
		}
		else if(!strcasecmp(dummy,"nb_fin_en_HERWIG"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%i",&(nb_fin_en_HERWIG));
		}
		else if(!strcasecmp(dummy,"nb_init_part_HERWIG"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%i",&(nb_init_part_HERWIG));
		}
		else if(!strcasecmp(dummy,"nb_fin_part_HERWIG"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%i",&(nb_fin_part_HERWIG));
		}
		else if(!strcasecmp(dummy,"Emin_hadro_PYTHIA_new"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%lf",&Emin_hadro_PYTHIA_new);
		}
		else if(!strcasecmp(dummy,"Emax_hadro_PYTHIA_new"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%lf",&Emax_hadro_PYTHIA_new);
		}
		else if(!strcasecmp(dummy,"nb_init_en_PYTHIA_new"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%i",&nb_init_en_PYTHIA_new);
		}
		else if(!strcasecmp(dummy,"nb_fin_en_PYTHIA_new"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%i",&nb_fin_en_PYTHIA_new);
		}
		else if(!strcasecmp(dummy,"nb_init_part_PYTHIA_new"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%i",&nb_init_part_PYTHIA_new);
		}
		else if(!strcasecmp(dummy,"nb_fin_part_PYTHIA_new"))
		{
			fscanf(param_file,"%s",dummy);
			fscanf(param_file,"%i",&nb_fin_part_PYTHIA_new);
		}*/
	}
	fclose(param_file);
	
	if(session == 0){
		if(!(read_fM_infos(parameters) == 1)){
			return 0; // there was an error in the fM_tables infos file
		}
	}
	
	if(!(read_gamma_infos(parameters) == 1)){
		return 0; // there was an error in the gamma_tables infos file
	}
	
	if(parameters->primary_only == 0){
		if(!(read_hadronization_infos(parameters)) == 1){
			return 0; // there was an error in the hadronization tables infos file
		}
	}
	
	if(parameters->spectrum_choice==1)
	{
		parameters->amplitude=amplitude_lognormal;
		parameters->stand_dev=stand_dev_lognormal;
		parameters->crit_mass=crit_mass_lognormal;
		
		parameters->amplitude = parameters->amplitude*mass_conversion/pow(leng_conversion,3.); // conversion from CGS to GeV
	}
	else if(parameters->spectrum_choice==2)
	{
		parameters->amplitude=amplitude_powerlaw;
		parameters->eqstate=eqstate_powerlaw;
		
		double gamma = 2.*parameters->eqstate/(1.+parameters->eqstate);
		parameters->amplitude = parameters->amplitude*pow(mass_conversion,gamma-1.)/pow(leng_conversion,3.); // conversion from CGS to GeV
	
	}
	else if(parameters->spectrum_choice==3)
	{
		parameters->amplitude=amplitude_critical_collapse;
		parameters->crit_mass=crit_mass_critical_collapse;
		parameters->amplitude = parameters->amplitude*pow(mass_conversion,-2.85)/pow(leng_conversion,3.); // conversion from CGS to GeV
	}
	else if(parameters->spectrum_choice == 5)
	{
		parameters->amplitude = amplitude_uniform/pow(leng_conversion,3.); // conversion from CGS to GeV
	}
		
	if(parameters->spectrum_choice==1 && parameters->stand_dev < 0.)
	{
		printf("\n\t [read_params] : ERROR WRONG MASS SPECTRUM STANDARD DEVIATION !\n");
		fflush(stdout);
		fclose(param_file);
		return 0;
	}
	
	if((parameters->spectrum_choice==1 || parameters->spectrum_choice==3) && parameters->crit_mass < 0.)
	{
		printf("\n\t [read_params] : ERROR WRONG MASS SPECTRUM CRITICAL MASS !\n");
		fflush(stdout);
		fclose(param_file);
		return 0;
	}
	parameters->crit_mass = parameters->crit_mass*mass_conversion; // conversion from CGS to GeV
	
	if(parameters->amplitude < 0. && (parameters->spectrum_choice == 1 || parameters->spectrum_choice == 2 || parameters->spectrum_choice == 3 || parameters->spectrum_choice == 5)){
		printf("\n\t [read_params] : ERROR WRONG MASS SPECTRUM AMPLITUDE !\n");
		fflush(stdout);
		fclose(param_file);
		return 0;
	}
	
	if(parameters->spectrum_choice_a == 2){
		parameters->stand_dev_a = stand_dev_a_gaussian;
		if(parameters->stand_dev_a < 0.){
			printf("\n\t [read_params] : ERROR WRONG SPIN SPECTRUM STANDARD DEVIATION !\n");
			fflush(stdout);
			return 0;
		}
		parameters->mean_a = mean_a_gaussian;
	}
	
	if(parameters->BHnumber > 1 && parameters->Mmin > parameters->Mmax)
	{
		printf("\n\t [read_params] : ERROR WRONG MASS SPECTRUM BOUNDARIES !\n");
		fflush(stdout);
		return 0;
	}
	
	if(session == 0)
	{
		if(parameters->Mmin < parameters->Mmin_fM || parameters->Mmax > parameters->Mmax_fM)
		{
			printf("\n\t [read_params] : ERROR MASS BOUNDARIES NOT COMPATIBLE WITH THE EVOLUTION TABLES !");
			printf("\n\t [read_params] : TABLES WERE COMPUTED FOR %.2e g < M < %.2e g\n",parameters->Mmin_fM/mass_conversion,parameters->Mmax_fM/mass_conversion);
			fflush(stdout);
			return 0;
		}
			
		if(parameters->amin < parameters->amin_fM || parameters->amax > parameters->amax_fM)
		{
			printf("\n\t [read_params] : ERROR SPIN BOUNDS NOT COMPATIBLE WITH THE EVOLUTION TABLES !");
			printf("\n\t [read_params] : EVOLUTION TABLES WERE COMPUTED FOR %.5f g < a < %.5f g\n",parameters->amin_fM,parameters->amax_fM);
			fflush(stdout);
			return 0;
		}
	}
	
	if(parameters->Emin < 0. || parameters-> Emax < 0. || parameters->Emax < parameters->Emin)
	{
		printf("\n\t [read_params] : ERROR WRONG ENERGY BOUNDARIES !\n");
		fflush(stdout);
		exit(0);
	}
	
	if(parameters->primary_only == 0)
	{			
		if(parameters->Emin < parameters->Emin_hadro || parameters->Emax > parameters->Emax_hadro)
		{
			if(parameters->hadronization_choice==0)
			{
				printf("\n\n\t [read_params] : WARNING ENERGY BOUNDARIES ARE NOT COMPATIBLE WITH THE PYTHIA HADRONIZATION TABLES !");
				printf("\n\t [read_params] : PYTHIA HADRONIZATION TABLES WERE COMPUTED FOR %.2e GEV < E < %.2e GEV",parameters->Emin_hadro,parameters->Emax_hadro);
				printf("\n\t [read_params] : EXTRAPOLATION WILL BE USED\n\n");
				fflush(stdout);
			}
			else if(parameters->hadronization_choice==1)
			{
				printf("\n\n\t [read_params] : WARNING ENERGY BOUNDARIES ARE NOT COMPATIBLE WITH THE HERWIG HADRONIZATION TABLES !");
				printf("\n\t [read_params] : HERWIG HADRONIZATION TABLES WERE COMPUTED FOR %.2e GeV < E < %.2e GeV",parameters->Emin_hadro,parameters->Emax_hadro);
				printf("\n\t [read_params] : EXTRAPOLATION WILL BE USED\n\n");
				fflush(stdout);
			}
			else if(parameters->hadronization_choice==2)
			{
				printf("\n\n\t [read_params] : WARNING ENERGY BOUNDARIES ARE NOT COMPATIBLE WITH THE NEW PYTHIA HADRONIZATION TABLES !");
				printf("\n\t [read_params] : NEW PYTHIA HADRONIZATION TABLES WERE COMPUTED FOR %.2e GeV < E < %.2e GeV",parameters->Emin_hadro,parameters->Emax_hadro);
				printf("\n\t [read_params] : EXTRAPOLATION WILL BE USED\n\n");
				fflush(stdout);
			}
		}
		
	}
				
	if(parameters->spectrum_choice == 0 && parameters->BHnumber > 1)
	{
		printf("\n\t [read_params] : WARNING THE DIRAC SPECTRUM WORKS ONLY WITH ONE BH MASS !\n");
		fflush(stdout);
		parameters->BHnumber = 1;
	}
	if(parameters->spectrum_choice_a == 0 && parameters->anumber > 1)
	{
		printf("\n\t [read_params] : WARNING THE DIRAC SPECTRUM WORKS ONLY WITH ONE BH SPIN !\n");
		fflush(stdout);
		parameters->anumber = 1;
	}
	
	return 1;
}

int read_fM_infos(struct param *parameters){
	// This function reads the informations about the f(M,a)/g(M,a) tables
	// and fills the parameters structure with the corresponding informations.
	
	char file_name[500];
	sprintf(file_name,"./src/tables/fM_tables/infos.txt");
	FILE *infos_file = fopen(file_name,"r");
	if(!infos_file){
		printf("\n\t [read_fM_infos] : ERROR COULD NOT OPEN FILE '%s' !\n",file_name);
		fflush(stdout);
		exit(0);
	}
	rewind(infos_file);
	
	char dummy[64];
	while(EOF != fscanf(infos_file,"%s",dummy))
	{
		if(!strncasecmp("#",dummy,1)) while ((EOF!=fscanf(infos_file,"%c",dummy))&&(strncasecmp("\n",dummy,1)));
		if(!strcasecmp(dummy,"Mmin_fM"))
		{
			fscanf(infos_file,"%s",dummy);
			fscanf(infos_file,"%lf",&(parameters->Mmin_fM));
		}
		else if(!strcasecmp(dummy,"Mmax_fM"))
		{
			fscanf(infos_file,"%s",dummy);
			fscanf(infos_file,"%lf",&(parameters->Mmax_fM));
		}
		else if(!strcasecmp(dummy,"amin_fM"))
		{
			fscanf(infos_file,"%s",dummy);
			fscanf(infos_file,"%lf",&(parameters->amin_fM));
		}
		else if(!strcasecmp(dummy,"amax_fM"))
		{
			fscanf(infos_file,"%s",dummy);
			fscanf(infos_file,"%lf",&(parameters->amax_fM));
		}
		else if(!strcasecmp(dummy,"nb_fM_masses"))
		{
			fscanf(infos_file,"%s",dummy);
			fscanf(infos_file,"%i",&(parameters->nb_fM_masses));
		}
		else if(!strcasecmp(dummy,"nb_fM_a"))
		{
			fscanf(infos_file,"%s",dummy);
			fscanf(infos_file,"%i",&(parameters->nb_fM_a));
		}
	}
	fclose(infos_file);
	
	parameters->Mmin_fM = parameters->Mmin_fM*mass_conversion;
	parameters->Mmax_fM = parameters->Mmax_fM*mass_conversion;
	
	if(parameters->Mmin_fM < 0. || parameters->Mmax_fM < 0. || parameters->Mmin_fM > parameters->Mmax_fM){
		printf("\n\t [read_fM_infos] : ERROR WRONG MASS BOUNDARIES !\n");
		fflush(stdout);
		return 0;
	}
	if(parameters->amin_fM < 0. || parameters->amin_fM > 1. || parameters->amax_fM < 0. || parameters->amax_fM > 1. || parameters->amin_fM > parameters->amax_fM){
		printf("\n\t [read_fM_infos] : ERROR WRING SPIN BOUNDARIES !\n");
		fflush(stdout);
		return 0;
	}
	if(parameters->nb_fM_masses < 1 || parameters->nb_fM_a < 1){
		printf("\n\t [read_fM_infos] : ERROR WRONG TABLE DIMENSIONS !\n");
		fflush(stdout);
		return 0;
	}
	
	return 1; // this is the only way of returning 1: there was no error in the parameters
}

int read_gamma_infos(struct param *parameters){
	// This function reads the informations about the greybody factors and fits tables
	// and fills the parameters structure with the relevant informations.
	
	char file_name[500];
	sprintf(file_name,"./src/tables/gamma_tables/infos.txt");
	FILE *infos_file = fopen(file_name,"r");
	if(!infos_file){
		printf("\n\t [read_gamma_infos] : ERROR COULD NOT OPEN FILE '%s' !\n",file_name);
		fflush(stdout);
		exit(0);
	}
	rewind(infos_file);
	
	char dummy[64];
	while(EOF != fscanf(infos_file,"%s",dummy))
	{
		if(!strncasecmp("#",dummy,1)) while ((EOF!=fscanf(infos_file,"%c",dummy))&&(strncasecmp("\n",dummy,1)));
		if(!strcasecmp(dummy,"particle_number"))
		{
			fscanf(infos_file,"%s",dummy);
			fscanf(infos_file,"%i",&(parameters->particle_number));
		}
		else if(!strcasecmp(dummy,"nb_gamma_a"))
		{
			fscanf(infos_file,"%s",dummy);
			fscanf(infos_file,"%i",&(parameters->nb_gamma_a));
		}
		else if(!strcasecmp(dummy,"nb_gamma_x"))
		{
			fscanf(infos_file,"%s",dummy);
			fscanf(infos_file,"%i",&(parameters->nb_gamma_x));
		}
	}
	fclose(infos_file);
	
	if(parameters->particle_number < 1){
		printf("\n\t [read_gamma_infos] : ERROR WRONG PARTICLE NUMBER !\n");
		fflush(stdout);
		return 0;
	}
	if(parameters->nb_gamma_a < 1 || parameters->nb_gamma_x < 1){
		printf("\n\t [read_gamma_infos] : ERROR WRONG TABLE DIMENSIONS !\n");
		fflush(stdout);
		return 0;
	}
	
	return 1; // this is the only way of returning 1: there was no error in the parameters
}

int read_hadronization_infos(struct param *parameters){
	// This function reads the informations about the hadronization tables
	// and fills the parameters structure with the relevant informations.
	
	char file_name[500];
	switch(parameters->hadronization_choice){
		case 0:{
			sprintf(file_name,"./src/tables/pythia_tables/infos.txt");
			break;
		}
		case 1:{
			sprintf(file_name,"./src/tables/herwig_tables/infos.txt");
			break;
		}
		case 2:{
			sprintf(file_name,"./src/tables/pythia_tables_new/infos.txt");
			break;
		}
		default:{
			printf("\n\t [read_hadronization_infos] : ERROR WRONG HADRONIZATION CHOICE !\n");
			fflush(stdout);
			exit(0);
			break;
		}
	}
	FILE *infos_file = fopen(file_name,"r");
	if(!infos_file){
		printf("\n\t [read_hadronization_infos] : ERROR COULD NOT OPEN FILE '%s' !\n",file_name);
		fflush(stdout);
		exit(0);
	}
	rewind(infos_file);
	
	char dummy[64];
	while(EOF != fscanf(infos_file,"%s",dummy))
	{
		if(!strncasecmp("#",dummy,1)) while ((EOF!=fscanf(infos_file,"%c",dummy))&&(strncasecmp("\n",dummy,1)));
		if(!strcasecmp(dummy,"Emin_hadro"))
		{
			fscanf(infos_file,"%s",dummy);
			fscanf(infos_file,"%lf",&(parameters->Emin_hadro));
		}
		else if(!strcasecmp(dummy,"Emax_hadro"))
		{
			fscanf(infos_file,"%s",dummy);
			fscanf(infos_file,"%lf",&(parameters->Emax_hadro));
		}
		else if(!strcasecmp(dummy,"nb_init_en"))
		{
			fscanf(infos_file,"%s",dummy);
			fscanf(infos_file,"%i",&(parameters->nb_init_en));
		}
		else if(!strcasecmp(dummy,"nb_fin_en"))
		{
			fscanf(infos_file,"%s",dummy);
			fscanf(infos_file,"%i",&(parameters->nb_fin_en));
		}
		else if(!strcasecmp(dummy,"nb_init_part"))
		{
			fscanf(infos_file,"%s",dummy);
			fscanf(infos_file,"%i",&(parameters->nb_init_part));
		}
		else if(!strcasecmp(dummy,"nb_fin_part"))
		{
			fscanf(infos_file,"%s",dummy);
			fscanf(infos_file,"%i",&(parameters->nb_fin_part));
		}
	}
	fclose(infos_file);
	
	if(parameters->Emin_hadro < 0. || parameters->Emax_hadro < 0. || parameters->Emin_hadro > parameters->Emax_hadro){
		printf("\n\t [read_hadronization_infos] : ERROR WRONG ENERGY BOUNDARIES !\n");
		fflush(stdout);
		return 0;
	}
	if(parameters->nb_init_en < 1 || parameters->nb_fin_en < 1){
		printf("\n\t [read_hadronization_infos] : ERROR WRONG TABLE DIMENSIONS !\n");
		fflush(stdout);
		return 0;
	}
	if(parameters->nb_init_part < 1 || parameters->nb_fin_part < 1){
		printf("\n\t [read_hadronization_infos] : ERROR WRONG NUMBER OF PARTICLES !\n");
		fflush(stdout);
		return 0;
	}
	
	return 1; // this is the only way of returning 1: there was no error in the parameters
}

int memory_estimation(struct param *parameters,int session){
	// This function computes and displays an estimation of the
	// used RAM and static memory throughout the desired run,
	// depending on the input parameters. The user has the choice
	// to cancel the run. This will return 0 and thus stop the
	// BlackHawk execution.
	
	double mean_evolution = 1000.; // mean evolution_length of one BH
	double RAM_use; // contains the RAM estimation, in Bytes
	double static_use; // contains the static memory estimation, in Bytes
	switch(session){
		case 0:{ // BlackHawk_tot
			double RAM_use_steps[3]; // 3 distinct steps
			// definition of the RAM occupied by the arrays
			double init_masses = 8.*(parameters->BHnumber);
			double init_spins = 8.*(parameters->anumber);
			double spec_table = 8.*(parameters->BHnumber)*(parameters->anumber);
			double fM_table = 8.*(parameters->nb_fM_a)*(parameters->nb_fM_masses);
			double gM_table = 8.*(parameters->nb_fM_a)*(parameters->nb_fM_masses);
			double fM_masses = 8.*(parameters->nb_fM_masses);
			double fM_a = 8.*(parameters->nb_fM_a);
			double evol_times = 8.*(parameters->BHnumber)*(parameters->anumber);
			double rank = 4.*(parameters->BHnumber)*(parameters->anumber);
			double sorted_times = 8.*(parameters->BHnumber)*(parameters->anumber);
			double life_masses = 8.*(parameters->BHnumber)*(parameters->anumber)*(parameters->limit);
			double life_spins = 8.*(parameters->BHnumber)*(parameters->anumber)*(parameters->limit);
			double life_times = 8.*(parameters->limit);
			double dts = 8.*(parameters->limit);
			double gammas = 8.*4.*(parameters->nb_gamma_a)*(parameters->nb_gamma_x);
			double gamma_a = 8.*(parameters->nb_gamma_a);
			double gamma_x = 8.*(parameters->nb_gamma_x);
			double fits = 8.*4.*(parameters->nb_gamma_a)*7.;
			double dof = 8.*(parameters->particle_number + parameters->grav);
			double masses_primary = 8.*(parameters->particle_number + parameters->grav);
			double masses_secondary = 8.*(parameters->nb_fin_part);
			double spins = 8.*(parameters->particle_number + parameters->grav);
			double times = 8.*mean_evolution*(parameters->BHnumber)*(parameters->anumber);
			double energies = 8.*(parameters->Enumber);
			double tables = 8.*(parameters->nb_fin_part)*(parameters->nb_init_en)*(parameters->nb_fin_en)*(parameters->nb_fin_part);
			double initial_energies = 8.*(parameters->nb_init_en);
			double final_energies = 8.*(parameters->nb_fin_en);
			double partial_hadronized_spectra = 8.*(parameters->nb_fin_part)*(parameters->Enumber)*(parameters->nb_fin_en);
			double partial_primary_spectra = 8.*(parameters->particle_number + parameters->grav)*(parameters->Enumber);
			double partial_integrated_hadronized_spectra = 8.*(parameters->nb_fin_part)*(parameters->nb_fin_en);
			RAM_use_steps[0] = init_masses + init_spins + spec_table; // step 1
			RAM_use_steps[1] = init_masses + init_spins + spec_table + fM_table + fM_a + fM_masses + gM_table + evol_times + rank + sorted_times + life_masses + life_spins + life_times + dts; // step 2
			RAM_use_steps[2] = spec_table + life_masses + life_spins + life_times + dts + gammas + gamma_a + gamma_x + fits + dof + spins + masses_primary + times + energies + partial_primary_spectra; // step 3
			if(parameters->primary_only == 0){
				RAM_use_steps[2] += tables + initial_energies + final_energies + partial_hadronized_spectra + partial_integrated_hadronized_spectra + masses_secondary; // step 4
			}
			RAM_use = RAM_use_steps[ind_max(RAM_use_steps,3)]; // finding the maximum use
			// definition of the disk memory used by the output files
			double BH_spectrum = 12.*(parameters->BHnumber + 1.)*(parameters->anumber + 1.);
			double life_evolutions = 12.*(parameters->BHnumber)*(1. + 2.*parameters->anumber)*(parameters->BHnumber)*(parameters->anumber)*mean_evolution;
			double primary_spectrum = 12.*(parameters->particle_number + parameters->grav)*(parameters->Enumber)*mean_evolution*(parameters->BHnumber)*(parameters->anumber);
			double secondary_spectrum;
			if(parameters->primary_only == 0){
				secondary_spectrum = 12.*(parameters->nb_fin_part)*(parameters->nb_fin_en)*mean_evolution*(parameters->BHnumber)*(parameters->anumber);
			}
			else{ // skipping hadronization
				secondary_spectrum = 0.;
			}
			static_use = BH_spectrum + life_evolutions + primary_spectrum + secondary_spectrum;
			break;
		}
		case 1:{ // session 1
			double RAM_use_steps[4]; // 4 distinctive steps
			double BH_masses = 8.*(parameters->BHnumber);
			double BH_spins = 8.*(parameters->anumber);
			double spec_table = 8.*(parameters->BHnumber)*(parameters->anumber);
			double gammas = 8.*4.*(parameters->nb_gamma_a)*(parameters->nb_gamma_x);
			double gamma_a = 8.*(parameters->nb_gamma_a);
			double gamma_x = 8.*(parameters->nb_gamma_x);
			double fits = 8.*4.*(parameters->nb_gamma_a)*7.;
			double dof = 8.*(parameters->particle_number + parameters->grav);
			double masses1 = 8.*(parameters->particle_number + parameters->grav);
			double masses2 = 8.*(parameters->nb_fin_part);
			double spins = 8.*(parameters->particle_number + parameters->grav);
			double instantaneous_primary_spectra = 8.*(parameters->particle_number + parameters->grav)*(parameters->Enumber);
			double energies = 8.*(parameters->Enumber);
			double tables = 8.*(parameters->nb_fin_part)*(parameters->nb_init_en)*(parameters->nb_fin_en)*(parameters->nb_fin_part);
			double initial_energies = 8.*(parameters->nb_init_en);
			double final_energies = 8.*(parameters->nb_fin_en);
			double instantaneous_hadronized_spectra = 8.*(parameters->nb_fin_part)*(parameters->Enumber)*(parameters->nb_fin_en);
			double instantaneous_integrated_hadronized_spectra = 8.*(parameters->nb_fin_part)*(parameters->nb_fin_en);
			RAM_use_steps[0] = BH_masses + BH_spins + spec_table; // step 1
			RAM_use_steps[1] = BH_masses + spec_table + gammas + gamma_a + gamma_x + fits + dof + spins + masses1 + instantaneous_primary_spectra + energies; // step 2
			if(parameters->primary_only == 0){
				RAM_use_steps[2] = instantaneous_primary_spectra + energies + tables + initial_energies + final_energies + masses2 + instantaneous_hadronized_spectra; // step 3
				RAM_use_steps[3] = instantaneous_primary_spectra + energies + initial_energies + final_energies + instantaneous_hadronized_spectra + instantaneous_integrated_hadronized_spectra; // step 4
			}
			else{ // skipping hadronization
				RAM_use_steps[2] = 0.;
				RAM_use_steps[3] = 0.;
			}
			RAM_use = RAM_use_steps[ind_max(RAM_use_steps,4)]; // finding the maximum use
			// definition of the disk memory used by the output files
			double BH_spectrum = 12.*(parameters->BHnumber + 1.)*(parameters->anumber +1.);
			double primary_spectrum = 12.*(parameters->Enumber)*(parameters->particle_number + parameters->grav);
			double secondary_spectrum;
			if(parameters->primary_only == 0){
				secondary_spectrum = 12.*(parameters->nb_fin_en)*(parameters->nb_fin_part);
			}
			else{ // skipping hadronization
				secondary_spectrum = 0.;
			}
			static_use = BH_spectrum + primary_spectrum + secondary_spectrum;
			break;
		}
		
		default:{
			printf("\n\t [memory_estimation] : ERROR WRONG SESSION !\n");
			fflush(stdout);
			exit(0);
			break;
		}
	}
	if(parameters->full_output){
		printf("\n\t Running this session will use around %.3f MB of RAM and %.3f MB of disc memory.\n",RAM_use/1.e+6,static_use/1.e+6); // conversion to MB
		printf("\n\t Do you want to continue? (type y or n)\t");
		fflush(stdout);
		char answer;
		scanf("%c",&answer);
		if(answer == 'y'){ // in this case the run will continue
			printf("\n");
			return 1;
		}
		else if(answer == 'n'){ // in this case the run will stop
			return 0;
		}
		else{
			printf("\n\t [memory_estimation] : ERROR WRONG ANSWER !\n");
			fflush(stdout);
			exit(0);
		}
	}
	else{
		return 1;
	}
}
